package com.example.kru13.hledanimin;

import android.app.Activity;
import android.content.Context;
import android.util.Log;
import android.widget.ImageButton;

import java.util.Random;

/**
 * Created by kru13 on 20.10.16.
 */

public class Game {

    private final String TAG = "Game";
    private int[] solution = new int[100];
    private int[] board = new int[100];
    private boolean isEnd = false;

    public static final int FLAG = 0;
    public static final int MINE = 1;
    public static final int EXPLODED = 2;

    Activity context;

    public Game(Activity context) {
        this.context = context;
    }

    public void genPole(int count) {

        for (int i = 0; i < 10; i++)
            for (int j = 0; j < 10; j++) {
                setSolution(i,j,0);
            }

        Random rnd = new Random();

        for (int i = 0; i < count; i++) {
            int x = rnd.nextInt(10);
            int y = rnd.nextInt(10);
            while ( getSolution(x,y) < 0 ) {
                x = rnd.nextInt(10);
                y = rnd.nextInt(10);
            }
            setSolution(x,y,-1);
        }

        for (int i = 0; i < 10; i++)
            for (int j = 0; j < 10; j++) {
                setSolution(i,j,getMineCount(i,j));
                setBoard(i,j,-10);
            }
    }

    protected void endGame() {

        isEnd = true;
        for (int i = 0; i < 10; i++)
            for (int j = 0; j < 10; j++) {
                if (getBoard(i,j) != -2)
                    setBoard(i,j,getSolution(i,j));
            }

        context.runOnUiThread(new Runnable() {
            public void run() {
                ImageButton img = (ImageButton)context.findViewById(R.id.imageButton);
                img.setImageResource(R.drawable.smiley2);
            }
        });
    }

    protected void mark(int x, int y) {
        if (isEnd == true) return;
        if (getBoard(x,y) != -3 && getBoard(x,y) != -10) return;

        if (getBoard(x, y) == -3)
            setBoard(x,y,-10);
        else setBoard(x,y,-3);

    }

    protected void unHide(int x, int y) {

        if (isEnd == true) return;
        if (getBoard(x,y) == -3) return;

        Log.d(TAG,"Unhide " + x + " " + y);

        if (getSolution(x, y) == -1) {setBoard(x, y, -2); endGame(); return;}
        if (getSolution(x, y) > 0 ) {setBoard(x, y, getSolution(x,y));  return; }

        if (getBoard(x, y) == -10) {
            setBoard(x,y,0);
            if (x > 0) unHide(x-1,y);
            if (x < 9) unHide(x+1,y);
            if (y > 0) unHide(x,y-1);
            if (y < 9) unHide(x,y+1);
            if (x > 0 && y > 0) unHide(x-1,y-1);
            if (x < 9 && y > 0) unHide(x+1,y-1);
            if (x > 0 && y < 9) unHide(x-1,y+1);
            if (x < 9 && y < 9) unHide(x+1,y+1);
        }

    }

    protected int getMineCount(int x, int y) {
        int c = 0;
        if (getSolution(x,y) == -1) return -1;
        for (int i = -1; i < 2; i++)
            for (int j = -1; j < 2; j++) {
                if ( x + i < 0) continue;
                if ( y + j < 0) continue;
                if ( x + i > 9) continue;
                if ( y + j > 9) continue;
                if (getSolution(x+i, y+j) == -1) c++;
            }
        return c;
    }

    protected int getSolution(int x, int y) {
        return solution[y*10+x];
    }

    protected void setSolution(int x, int y, int val) {
        solution[y*10+x] = val;
    }

    protected int getBoard(int x, int y) {
        return board[y*10+x];
    }

    protected void setBoard(int x, int y, int val) {
        board[y*10+x] = val;
    }



}
