package com.example.kru13.hledanimin;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

/**
 * Created by kru13 on 20.10.16.
 */

public class GameView extends View {

    private Game game;

    private float width;
    private float height;
    private long startTime;

    private Bitmap bmp[];
    int size = 10;

    private final String TAG = "GameView";

    public GameView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    public GameView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public GameView(Context context) {
        super(context);
        init();
    }

    private void init()
    {
        // Load bitmaps in advance and reuse them for onDraw()
        bmp = new Bitmap[3];
        bmp[Game.MINE] = BitmapFactory.decodeResource(getResources(), R.drawable.mine);
        bmp[Game.EXPLODED] = BitmapFactory.decodeResource(getResources(), R.drawable.mine3);
        bmp[Game.FLAG] = BitmapFactory.decodeResource(getResources(), R.drawable.flag);

    }

    public void initGame(Game context) {
        this.game = context;
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        width = w / size;
        height = h / size;
        Log.d(TAG, "onSizeChanged: width " + width + ", height " + height);

        super.onSizeChanged(w, h, oldw, oldh);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch(event.getAction() & MotionEvent.ACTION_MASK)
        {
            case MotionEvent.ACTION_DOWN:
                startTime = System.currentTimeMillis();
                break;
            // measure time difference between ACTION_DOWN and ACTION_UP and decide whether it is a long/short touch
            case MotionEvent.ACTION_UP:
                long time = System.currentTimeMillis() - startTime;
                if (time < 200) {
                    select((int) (event.getX() / width), (int) (event.getY() / height), 0);

                } else {
                    select((int) (event.getX() / width), (int) (event.getY() / height), 1);
                }
        }
        return true;
    }

    private void select(int x, int y, int type) {

        if (type == 0) this.game.unHide(x, y);
        if (type == 1) this.game.mark(x, y);

        this.invalidate();
    }

    @Override
    protected void onDraw(Canvas canvas) {
        // Draw the background...
        Paint background = new Paint();
        background.setColor(getResources().getColor(R.color.minesweeper_background));
        canvas.drawRect(0, 0, getWidth(), getHeight(), background);

        //draw lines
        Paint dark = new Paint();
        dark.setColor(getResources().getColor(R.color.minesweeper_dark));
        Paint hilite = new Paint();
        hilite.setColor(getResources().getColor(R.color.minesweeper_hilite));
        Paint light = new Paint();
        light.setColor(getResources().getColor(R.color.minesweeper_light));

        for (int i = 0; i < 10; i++) {
            canvas.drawLine(0, i * height, getWidth(), i * height,light);
            canvas.drawLine(0, i * height + 1, getWidth(), i * height + 1, light);
            canvas.drawLine(i * width, 0, i * width, getHeight(), light);
            canvas.drawLine(i * width + 1, 0, i * width + 1, getHeight(), light);
        }

        //draw text
        Paint foreground = new Paint(Paint.ANTI_ALIAS_FLAG);
        foreground.setColor(getResources().getColor(R.color.minesweeper_foreground));
        foreground.setStyle(Paint.Style.FILL);
        foreground.setTextSize(height * 0.75f);
        foreground.setTextScaleX(width / height);
        foreground.setTextAlign(Paint.Align.CENTER);

        // Draw the number in the center of the tile
        Paint.FontMetrics fm = foreground.getFontMetrics();
        // Centering in X: use alignment (and X at midpoint)
        float x = width / 2;
        // Centering in Y: measure ascent/descent first
        float y = height / 2 - (fm.ascent + fm.descent) / 2;

        if (game == null) return;

        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {

                if (this.game.getBoard(i,j) > 0)
                    canvas.drawText(String.valueOf(this.game.getBoard(i, j)), i * width + x, j * height + y, foreground);

                if (this.game.getBoard(i, j) == -10)
                    canvas.drawRect(i*(int)width,j*(int)height,
                            (i+1)*(int)width,(j+1)*(int)height,dark);

                if (this.game.getBoard(i, j) == 0)
                    canvas.drawRect(i*(int)width,j*(int)height,
                            (i+1)*(int)width,(j+1)*(int)height,light);

                if (this.game.getBoard(i,j) == -1) {
                    canvas.drawBitmap(bmp[Game.MINE], new Rect(0,0,bmp[Game.MINE].getWidth(),bmp[Game.MINE].getHeight()),
                            new Rect(i*(int)width,j*(int)height,
                                    (i+1)*(int)width,(j+1)*(int)height),foreground);
                }
                if (this.game.getBoard(i,j) == -2) {
                    canvas.drawBitmap(bmp[Game.EXPLODED], new Rect(0,0,bmp[Game.EXPLODED].getWidth(),bmp[Game.EXPLODED].getHeight()),
                            new Rect(i*(int)width,j*(int)height,
                                    (i+1)*(int)width,(j+1)*(int)height),foreground);
                }
                if (this.game.getBoard(i,j) == -3) {
                    canvas.drawBitmap(bmp[Game.FLAG], new Rect(0,0,bmp[Game.FLAG].getWidth(),bmp[Game.FLAG].getHeight()),
                            new Rect(i*(int)width,j*(int)height,
                                    (i+1)*(int)width,(j+1)*(int)height),foreground);
                }
            }
        }
    }


}

